@@ -4,30 +4,33 @@ use strict;
 use warnings;
 use vars qw( $VERSION @EXPORT @EXPORT_OK @ISA $CurrentPackage @IncludeLibs $ScanFileRE );
 
-$VERSION   = '1.24';
+$VERSION   = '1.30';
 @EXPORT    = qw( scan_deps scan_deps_runtime );
 @EXPORT_OK = qw( scan_line scan_chunk add_deps scan_deps_runtime path_to_inc_name );
 
 use Config;
 require Exporter;
 our @ISA = qw(Exporter);
-use constant dl_ext  => ".$Config{dlext}";
-use constant lib_ext => $Config{lib_ext};
 use constant is_insensitive_fs => (
-    -s $0 
+    -s $0
         and (-s lc($0) || -1) == (-s uc($0) || -1)
         and (-s lc($0) || -1) == -s $0
 );
 
 use version;
-use Cwd ();
 use File::Path ();
 use File::Temp ();
-use File::Spec ();
-use File::Basename ();
 use FileHandle;
 use Module::Metadata;
 
+# NOTE: Keep the following imports exactly as specified, even if the Module::ScanDeps source
+# doesn't reference some of them. See '"use lib" idioms' for the reason.
+use Cwd (qw(abs_path));
+use File::Spec;
+use File::Spec::Functions;
+use File::Basename;
+
+
 $ScanFileRE = qr/(?:^|\\|\/)(?:[^.]*|.*\.(?i:p[ml]|t|al))$/;
 
 =head1 NAME
@@ -174,7 +177,7 @@ returns a reference to it.
     $perl_name = path_to_inc_name($path, $warn)
 
 Assumes C<$path> refers to a perl file and does it's best to return the
-name as it would appear in %INC. Returns undef if no match was found 
+name as it would appear in %INC. Returns undef if no match was found
 and a prints a warning to STDERR if C<$warn> is true.
 
 E.g. if C<$path> = perl/site/lib/Module/ScanDeps.pm then C<$perl_name>
@@ -189,8 +192,8 @@ which to search modules without modifying C<@INC> itself.
 
 =head2 B<$Module::ScanDeps::ScanFileRE>
 
-You can set this global variable to specify a regular expression to 
-identify what files to scan. By default it includes all files of 
+You can set this global variable to specify a regular expression to
+identify what files to scan. By default it includes all files of
 the following types: .pm, .pl, .t and .al. Additionally, all files
 without a suffix are considered.
 
@@ -226,7 +229,7 @@ my %Preload = (
     'AnyDBM_File.pm'                    => [qw( SDBM_File.pm )],
     'AnyEvent.pm'                       => 'sub',
     'Authen/SASL.pm'                    => 'sub',
-    'B/Hooks/EndOfScope.pm'             => 
+    'B/Hooks/EndOfScope.pm'             =>
         [qw( B/Hooks/EndOfScope/PP.pm B/Hooks/EndOfScope/XS.pm )],
     'Bio/AlignIO.pm'                    => 'sub',
     'Bio/Assembly/IO.pm'                => 'sub',
@@ -257,7 +260,7 @@ my %Preload = (
                 _glob_in_inc('Catalyst/DispatchType', 1));
     },
     'Catalyst/Engine.pm'                => 'sub',
-    'CGI/Application/Plugin/Authentication.pm' => 
+    'CGI/Application/Plugin/Authentication.pm' =>
         [qw( CGI/Application/Plugin/Authentication/Store/Cookie.pm )],
     'CGI/Application/Plugin/AutoRunmode.pm' => [qw( Attribute/Handlers.pm )],
     'charnames.pm'                      => \&_unicore,
@@ -271,7 +274,7 @@ my %Preload = (
     'Crypt/Random/Generator.pm'         => sub {
         _glob_in_inc('Crypt/Random/Provider', 1);
     },
-    'Date/Manip.pm'                     => 
+    'Date/Manip.pm'                     =>
         [qw( Date/Manip/DM5.pm Date/Manip/DM6.pm )],
     'Date/Manip/Base.pm'                => sub {
         _glob_in_inc('Date/Manip/Lang', 1);
@@ -292,7 +295,7 @@ my %Preload = (
     'DBIx/Perlish.pm'                   => [qw( attributes.pm )],
     'DBIx/ReportBuilder.pm'             => 'sub',
     'Device/ParallelPort.pm'            => 'sub',
-    'Device/SerialPort.pm'              => 
+    'Device/SerialPort.pm'              =>
         [qw( termios.ph asm/termios.ph sys/termiox.ph sys/termios.ph sys/ttycom.ph )],
     'diagnostics.pm'                    => sub {
         # shamelessly taken and adapted from diagnostics.pm
@@ -314,7 +317,7 @@ my %Preload = (
         ) {
             return $_ if _find_in_inc($_);
         }
-        
+
         for (
               "$archlib/pods/perldiag.pod",
               "$privlib/pods/perldiag-$Config{version}.pod",
@@ -332,6 +335,7 @@ my %Preload = (
     'ExtUtils/MakeMaker.pm'             => sub {
         grep /\bMM_/, _glob_in_inc('ExtUtils', 1);
     },
+    'FFI/Platypus.pm'                   => 'sub',
     'File/Basename.pm'                  => [qw( re.pm )],
     'File/BOM.pm'                       => [qw( Encode/Unicode.pm )],
     'File/HomeDir.pm'                   => 'sub',
@@ -340,6 +344,7 @@ my %Preload = (
         map { my $name = $_; $name =~ s!::!/!g; "$name.pm" } @File::Spec::ISA;
     },
     'Gtk2.pm'                           => [qw( Cairo.pm )], # Gtk2.pm does: eval "use Cairo;"
+    'HTTP/Entity/Parser.pm'             => 'sub',
     'HTTP/Message.pm'                   => [qw( URI/URL.pm URI.pm )],
     'Image/ExifTool.pm'                 => sub {
         return(
@@ -364,6 +369,9 @@ my %Preload = (
         # but accept JSON::XS, too (because JSON.pm might use it if present)
         return( grep /^JSON\/(PP|XS)/, _glob_in_inc('JSON', 1) );
     },
+    'JSON/MaybeXS.pm'                   => [qw(
+        Cpanel/JSON/XS.pm JSON/XS.pm JSON/PP.pm
+    )],
     'List/MoreUtils.pm'                 => 'sub',
     'List/SomeUtils.pm'                 => 'sub',
     'Locale/Maketext/Lexicon.pm'        => 'sub',
@@ -382,7 +390,7 @@ my %Preload = (
     },
     'LWP/Parallel/UserAgent.pm'         => [qw( LWP/Parallel.pm )],
     'LWP/UserAgent.pm'                  => sub {
-        return( 
+        return(
           qw( URI/URL.pm URI/http.pm LWP/Protocol/http.pm ),
           _glob_in_inc("LWP/Authen", 1),
           _glob_in_inc("LWP/Protocol", 1),
@@ -520,13 +528,13 @@ my %Preload = (
         _glob_in_inc('XMLRPC/Transport', 1);
     },
     'YAML.pm'                           => [qw( YAML/Loader.pm YAML/Dumper.pm )],
-    'YAML/Any.pm'                       => sub { 
+    'YAML/Any.pm'                       => sub {
         # try to figure out what YAML::Any would have used
         my $impl = eval "use YAML::Any; YAML::Any->implementation;";
-        unless ($@) 
-        { 
-            $impl =~ s!::!/!g; 
-            return "$impl.pm"; 
+        unless ($@)
+        {
+            $impl =~ s!::!/!g;
+            return "$impl.pm";
         }
         _glob_in_inc('YAML', 1);        # fallback
     },
@@ -613,7 +621,51 @@ sub scan_deps {
         }
     }
 
-    scan_deps_static(\%args);
+    {
+        ## "use lib" idioms
+        #
+        # We want to correctly interprete stuff like
+        #
+        #   use FindBin;
+        #   use lib "$FindBin/../lib";
+        #
+        # Find out what $FindBin::Bin etc would have been if "use FindBin" had been
+        # called in the first file to analyze.
+        #
+        # Notes:
+        # (1) We don't want to reimplement FindBin, hence fake $0 locally (as the path of the
+        #     first file analyzed) and call FindBin::again().
+        # (2) If the caller of scan_deps() itself uses FindBin, we don't want to overwrite
+        #     the value of "their" $FindBin::Bin.
+        #
+        # Other idioms seen sometimes:
+        #
+        # use lib "$ENV{FOO}/path";
+        # use lib File::Spec->catdir($FindBin::Bin, qw[.. qqlib] );
+        # use lib catdir(dirname($0), "perl");
+        # use lib dirname(abs_path($0));
+        #
+        # In order to correctly interprete these, the modules referenced have to be imported.
+
+
+        require FindBin;
+
+        local $FindBin::Bin;
+        local $FindBin::RealBin;
+        local $FindBin::Script;
+        local $FindBin::RealScript;
+
+        my $_0 = $args{files}[0];
+        local *0 = \$_0;
+        FindBin->again();
+
+        our $Bin = $FindBin::Bin;
+        our $RealBin = $FindBin::RealBin;
+        our $Script = $FindBin::Script;
+        our $RealScript = $FindBin::RealScript;
+
+        scan_deps_static(\%args);
+    }
 
     if ($args{execute} or $args{compile}) {
         scan_deps_runtime(
@@ -675,7 +727,7 @@ sub scan_deps_static {
         }else{ # no caching callback given
             @pm = scan_file($file);
         }
-        
+
         foreach my $pm (@pm){
             add_deps(
                      used_by => $key,
@@ -701,7 +753,7 @@ sub scan_deps_static {
 
     # prevent utf8.pm from being scanned
     $_skip->{$rv->{"utf8.pm"}{file}}++ if $rv->{"utf8.pm"};
-   
+
     while ($recurse) {
         my $count = keys %$rv;
         my @files = sort grep { defined $_->{file} && -T $_->{file} } values %$rv;
@@ -736,16 +788,14 @@ sub scan_deps_runtime {
         foreach my $file (@$files) {
             next unless $file =~ $ScanFileRE;
 
-            my ($inchash, $dl_shared_objects, $incarray) = _compile_or_execute($file);
-            _merge_rv(_make_rv($inchash, $dl_shared_objects, $incarray), $rv);
+            _merge_rv(_info2rv(_compile_or_execute($file)), $rv);
         }
     }
     elsif ($execute) {
         foreach my $file (@$files) {
             $execute = [] unless ref $execute;  # make sure it's an array ref
 
-            my ($inchash, $dl_shared_objects, $incarray) = _compile_or_execute($file, $execute);
-            _merge_rv(_make_rv($inchash, $dl_shared_objects, $incarray), $rv);
+            _merge_rv(_info2rv(_compile_or_execute($file, $execute)), $rv);
         }
     }
 
@@ -798,20 +848,25 @@ sub scan_line {
     $line =~ s/\s*#.*$//;
     $line =~ s/[\\\/]+/\//g;
 
+  CHUNK:
     foreach (split(/;/, $line)) {
         s/^\s*//;
+        #  handle single line blocks like 'do { package foo; use xx; }'
+        s/^(?:do\s*)?\{\s*//;
+        s/\}$//;
 
         if (/^package\s+(\w+)/) {
             $CurrentPackage = $1;
             $CurrentPackage =~ s{::}{/}g;
-            return;
+            next CHUNK;
         }
         # use VERSION:
         if (/^(?:use|require)\s+v?(\d[\d\._]*)/) {
           # include feature.pm if we have 5.9.5 or better
           if (version->new($1) >= version->new("5.9.5")) {
               # seems to catch 5.9, too (but not 5.9.4)
-            return "feature.pm";
+            $found{"feature.pm"}++;
+            next CHUNK;
           }
         }
 
@@ -821,13 +876,13 @@ sub scan_line {
             # be specified for the "autouse" and "if" pragmas, e.g.
             #   use autouse Module => qw(func1 func2);
             #   use autouse "Module", qw(func1);
-            # To avoid to parse them ourself, we simply try to eval the 
+            # To avoid to parse them ourself, we simply try to eval the
             # string after the pragma (in a list context). The MODULE
             # should be the first ("autouse") or second ("if") element
             # of the list.
             my $module;
-            { 
-                no strict; no warnings; 
+            {
+                no strict; no warnings;
                 if ($pragma eq "autouse") {
                     ($module) = eval $args;
                 }
@@ -835,7 +890,7 @@ sub scan_line {
                     # The syntax of the "if" pragma is
                     #   use if COND, MODULE => ARGUMENTS
                     # The COND may contain undefined functions (i.e. undefined
-                    # in Module::ScanDeps' context) which would throw an 
+                    # in Module::ScanDeps' context) which would throw an
                     # exception. Sneak  "1 || " in front of COND so that
                     # COND will not be evaluated. This will work in most
                     # cases, but there are operators with lower precedence
@@ -846,7 +901,9 @@ sub scan_line {
                 return if $@ or !defined $module;
             };
             $module =~ s{::}{/}g;
-            return ("$pragma.pm", "$module.pm");
+            $found{"$pragma.pm"}++;
+            $found{"$module.pm"}++;
+            next CHUNK;
         }
 
         if (my ($how, $libs) = /^(use \s+ lib \s+ | (?:unshift|push) \s+ \@INC \s+ ,) (.+)/x)
@@ -856,13 +913,13 @@ sub scan_line {
             foreach my $dir (do { no strict; no warnings; eval $libs }) {
                 next unless defined $dir;
                 my @dirs = $dir;
-                push @dirs, "$dir/$ver", "$dir/$archname", "$dir/$ver/$archname" 
+                push @dirs, "$dir/$ver", "$dir/$archname", "$dir/$ver/$archname"
                     if $how =~ /lib/;
                 foreach (@dirs) {
                     unshift(@INC, $_) if -d $_;
                 }
             }
-            next;
+            next CHUNK;
         }
 
         $found{$_}++ for scan_chunk($_);
@@ -876,7 +933,7 @@ my %LoaderRegexp; # cache
 sub _build_loader_regexp {
     my $loaders = shift;
     my $prefix = (@_ && $_[0]) ? $_[0].'::' : '';
-   
+
     my $loader = join '|', map quotemeta($_), split /\s+/, $loaders;
     my $regexp = qr/^\s* use \s+ ($loader)(?!\:) \b \s* (.*)/sx;
     # WARNING: This doesn't take the prefix into account
@@ -910,6 +967,9 @@ sub scan_chunk {
     my $module = eval {
         $_ = $chunk;
         s/^\s*//;
+        #  handle single line blocks like 'do { package foo; use xx; }'
+        s/^(?:do\s*)?\{\s*//;
+        s/\}\s*$//;
 
         # TODO: There's many more of these "loader" type modules on CPAN!
         # scan for the typical module-loader modules
@@ -962,7 +1022,7 @@ sub scan_chunk {
             return \@mods if @mods;
         }
         if (/\b(?:en|de)code\(\s*['"]?([-\w]+)/) {
-            return [qw( Encode.pm ), _find_encoding($1)]; 
+            return [qw( Encode.pm ), _find_encoding($1)];
         }
 
         return $1 if /\b do \s+ ([\w:\.\-\\\/\"\']*)/x;
@@ -1013,10 +1073,10 @@ sub scan_chunk {
 }
 
 sub _find_encoding {
+    my ($enc) = @_;
     return unless $] >= 5.008 and eval { require Encode; %Encode::ExtModule };
 
-    my $mod = $Encode::ExtModule{ Encode::find_encoding($_[0])->name }
-      or return;
+    my $mod = eval { $Encode::ExtModule{ Encode::find_encoding($enc)->name } } or return;
     $mod =~ s{::}{/}g;
     return "$mod.pm";
 }
@@ -1031,7 +1091,7 @@ sub _add_info {
     $file = File::Spec->rel2abs($file);
     $file =~ s|\\|\/|go;
 
-    # Avoid duplicates that can arise due to case differences that don't actually 
+    # Avoid duplicates that can arise due to case differences that don't actually
     # matter on a case tolerant system
     if (File::Spec->case_tolerant()) {
         foreach my $key (keys %$rv) {
@@ -1065,7 +1125,7 @@ sub _add_info {
           if  ( (!File::Spec->case_tolerant() && !grep { $_ eq $used_by } @{ $rv->{$module}{used_by} })
              or ( File::Spec->case_tolerant() && !grep { lc($_) eq lc($used_by) } @{ $rv->{$module}{used_by} }));
 
-        # We assume here that another _add_info will be called to provide the other parts of $rv->{$used_by}    
+        # We assume here that another _add_info will be called to provide the other parts of $rv->{$used_by}
         push @{ $rv->{$used_by}{uses} }, $module
           if  ( (!File::Spec->case_tolerant() && !grep { $_ eq $module } @{ $rv->{$used_by}{uses} })
              or ( File::Spec->case_tolerant() && !grep { lc($_) eq lc($module) } @{ $rv->{$used_by}{uses} }));
@@ -1095,29 +1155,19 @@ sub add_deps {
             next;
         }
 
-        my $type = _gettype($file);
         _add_info( rv     => $rv,   module  => $module,
                    file   => $file, used_by => $used_by,
-                   type   => $type );
+                   type   => _gettype($file) );
 
-        if ($module =~ /(.*?([^\/]*))\.p[mh]$/i) {
-            my ($path, $basename) = ($1, $2);
+        if ((my $path = $module) =~ s/\.p[mh]$//i) {
 
             foreach (_glob_in_inc("auto/$path")) {
-                next if $_->{file} =~ m{\bauto/$path/.*/};  # weed out subdirs
-                next if $_->{name} =~ m{/\.(?:exists|packlist)$};
-                my ($ext,$type);
-                $ext = lc($1) if $_->{name} =~ /(\.[^.]+)$/;
-                if (defined $ext) {
-                    next if $ext eq lc(lib_ext());
-                    $type = 'shared' if $ext eq lc(dl_ext());
-                    $type = 'autoload' if ($ext eq '.ix' or $ext eq '.al');
-                }
-                $type ||= 'data';
+                next if $_->{name} =~ m{^auto/$path/.*/};  # weed out subdirs
+                next if $_->{name} =~ m{/(?:\.exists|\.packlist)$|\Q$Config{lib_ext}\E$};
 
                 _add_info( rv     => $rv,        module  => $_->{name},
                            file   => $_->{file}, used_by => $module,
-                           type   => $type );
+                           type   => _gettype($_->{name}) );
             }
 
             ### Now, handle module and distribution share dirs
@@ -1186,7 +1236,7 @@ sub _glob_in_inc {
 # like _glob_in_inc, but looks only at the first level
 # (i.e. the children of $subdir)
 # NOTE: File::Find has no public notion of the depth of the traversal
-# in its "wanted" callback, so it's not helpful 
+# in its "wanted" callback, so it's not helpful
 sub _glob_in_inc_1 {
     my $subdir  = shift;
     my $pm_only = shift;
@@ -1198,7 +1248,7 @@ sub _glob_in_inc_1 {
         my $dir = "$inc/$subdir";
         next unless -d $dir;
 
-        opendir my $dh, $dir or next; 
+        opendir(my $dh, $dir) or next;
         my @names = map { "$subdir/$_" } grep { -f "$dir/$_" } readdir $dh;
         closedir $dh;
 
@@ -1326,38 +1376,42 @@ sub add_preload_rule {
 sub _compile_or_execute {
     my ($file, $execute) = @_;
 
+    local $ENV{MSD_ORIGINAL_FILE} = $file;
+
     my ($ih, $instrumented_file) = File::Temp::tempfile(UNLINK => 1);
 
+    my (undef, $data_file) = File::Temp::tempfile(UNLINK => 1);
+    local $ENV{MSD_DATA_FILE} = $data_file;
+
     # spoof $0 (to $file) so that FindBin works as expected
     # NOTE: We don't directly assign to $0 as it has magic (i.e.
     # assigning has side effects and may actually fail, cf. perlvar(1)).
     # Instead we alias *0 to a package variable holding the correct value.
-    local $ENV{MSD_ORIGINAL_FILE} = $file;
     print $ih <<'...';
 BEGIN { my $_0 = $ENV{MSD_ORIGINAL_FILE}; *0 = \$_0; }
 ...
 
-    my (undef, $data_file) = File::Temp::tempfile(UNLINK => 1);
-    local $ENV{MSD_DATA_FILE} = $data_file;
-
     # NOTE: When compiling the block will run as the last CHECK block;
-    # when executing the block will run as the first END block and 
+    # when executing the block will run as the first END block and
     # the programs continues.
-    print $ih $execute ? "END\n" : "CHECK\n", <<'...';
+    print $ih
+        $execute ? "END\n" : "CHECK\n",
+        <<'...';
 {
-    # save %INC etc so that requires below don't pollute them
-    my %_INC = %INC;
-    my @_INC = @INC;
+    require DynaLoader;
     my @_dl_shared_objects = @DynaLoader::dl_shared_objects;
     my @_dl_modules = @DynaLoader::dl_modules;
 
-    require Cwd;
-    require DynaLoader;
-    require Data::Dumper;
-    require B; 
-    require Config;
+    # save %INC etc so that requires below don't pollute them
+    my %_INC = %INC;
+    my @_INC = @INC;
 
-    while (my ($k, $v) = each %_INC)
+    require Cwd;
+    require Data::Dumper;
+    require Config;
+    my $dlext = $Config::Config{dlext};
+
+    foreach my $k (keys %_INC)
     {
         # NOTES:
         # (1) An unsuccessful "require" may store an undefined value into %INC.
@@ -1368,6 +1422,8 @@ BEGIN { my $_0 = $ENV{MSD_ORIGINAL_FILE}; *0 = \$_0; }
         #     "Class/MOP/Class/Immutable/Moose/Meta/Class.pm" => "(set by Moose)"
         #     On some architectures (e.g. Windows) Cwd::abs_path() will throw
         #     an exception for such a pathname.
+
+        my $v = $_INC{$k};
         if (defined $v && !ref $v && -e $v)
         {
             $_INC{$k} = Cwd::abs_path($v);
@@ -1381,18 +1437,20 @@ BEGIN { my $_0 = $ENV{MSD_ORIGINAL_FILE}; *0 = \$_0; }
     # drop refs from @_INC
     @_INC = grep { !ref $_ } @_INC;
 
-    my $dlext = $Config::Config{dlext};
-    my @so = grep { defined $_ && -e $_ } Module::ScanDeps::DataFeed::_dl_shared_objects();
-    my @bs = @so;
-    my @shared_objects = ( @so, grep { s/\Q.$dlext\E$/\.bs/ && -e $_ } @bs );
+    my @dlls = grep { defined $_ && -e $_ } Module::ScanDeps::DataFeed::_dl_shared_objects();
+    my @shared_objects = @dlls;
+    push @shared_objects, grep { -e $_ } map { (my $bs = $_) =~ s/\.\Q$dlext\E$/.bs/; $bs } @dlls;
 
+    # write data file
     my $data_file = $ENV{MSD_DATA_FILE};
-    open my $fh, ">", $data_file 
+    open my $fh, ">", $data_file
         or die "Couldn't open $data_file: $!\n";
-    print $fh Data::Dumper->Dump(
-                  [    \%_INC,  \@_INC,   \@shared_objects    ], 
-                  [qw( *inchash *incarray *dl_shared_objects )]);
-    print $fh "1;\n";
+    print $fh Data::Dumper::Dumper(
+    {
+        '%INC'            => \%_INC,
+        '@INC'            => \@_INC,
+        dl_shared_objects => \@shared_objects,
+    });
     close $fh;
 
     sub Module::ScanDeps::DataFeed::_dl_shared_objects {
@@ -1411,7 +1469,6 @@ BEGIN { my $_0 = $ENV{MSD_ORIGINAL_FILE}; *0 = \$_0; }
         return if $mod eq 'B';
         return unless defined &{"$mod\::bootstrap"};
 
-        my $dl_ext = $Config::Config{dlext};
 
         # cf. DynaLoader.pm
         my @modparts = split(/::/, $mod);
@@ -1419,8 +1476,8 @@ BEGIN { my $_0 = $ENV{MSD_ORIGINAL_FILE}; *0 = \$_0; }
         my $modpname = join('/', @modparts);
 
         foreach my $dir (@_INC) {
-            my $file = "$dir/auto/$modpname/$modfname.$dl_ext";
-            return $file if -r $file;
+            my $file = "$dir/auto/$modpname/$modfname.$dlext";
+            return $file if -e $file;
         }
         return;
     }
@@ -1444,44 +1501,50 @@ BEGIN { my $_0 = $ENV{MSD_ORIGINAL_FILE}; *0 = \$_0; }
         $execute ? @$execute : ());
 
     die $execute
-        ? "SYSTEM ERROR in executing $file @$execute: $rc" 
-        : "SYSTEM ERROR in compiling $file: $rc" 
+        ? "SYSTEM ERROR in executing $file @$execute: $rc"
+        : "SYSTEM ERROR in compiling $file: $rc"
         unless $rc == 0;
-    
-    return _extract_info($data_file);
+
+    my $info = do $data_file
+        or die "error extracting info from -c/-x file: ", ($@ || "can't read $data_file: $!");
+
+    return $info;
 }
 
 # create a new hashref, applying fixups
-sub _make_rv {
-    my ($inchash, $dl_shared_objects, $inc_array) = @_;
+sub _info2rv {
+    my ($info) = @_;
 
     my $rv = {};
-    my @newinc = map(quotemeta($_), @$inc_array);
-    my $inc = join('|', sort { length($b) <=> length($a) } @newinc);
-    # don't pack lib/c:/ or lib/C:/
-    $inc = qr/$inc/i if(is_insensitive_fs());
+
+    my $incs = join('|', sort { length($b) <=> length($a) }
+                              map { s:\\:/:g; s:^(/.*?)/+$:$1:; quotemeta($_) }
+                                  @{ $info->{'@INC'} });
+    my $i = is_insensitive_fs() ? "i" : "";
+    my $strip_inc_prefix = qr{^(?$i:$incs)/};
 
     require File::Spec;
 
-    foreach my $key (keys(%$inchash)) {
-        my $newkey = $key;
-        $newkey =~ s"^(?:(?:$inc)/?)""sg if File::Spec->file_name_is_absolute($newkey);
+    foreach my $key (keys %{ $info->{'%INC'} }) {
+        (my $path = $info->{'%INC'}{$key}) =~ s:\\:/:g;
+        $key =~ s:\\:/:g;
+        $key =~ s/$strip_inc_prefix// if File::Spec->file_name_is_absolute($key);
 
-        $rv->{$newkey} = {
+        $rv->{$key} = {
             'used_by' => [],
-            'file'    => $inchash->{$key},
-            'type'    => _gettype($inchash->{$key}),
+            'file'    => $path,
+            'type'    => _gettype($path),
             'key'     => $key
         };
     }
 
-    foreach my $dl_file (@$dl_shared_objects) {
-        my $key = $dl_file;
-        $key =~ s"^(?:(?:$inc)/?)""s;
+    foreach my $path (@{ $info->{dl_shared_objects} }) {
+        $path =~ s:\\:/:g;
+        (my $key = $path) =~ s/$strip_inc_prefix//;
 
         $rv->{$key} = {
             'used_by' => [],
-            'file'    => $dl_file,
+            'file'    => $path,
             'type'    => 'shared',
             'key'     => $key
         };
@@ -1490,29 +1553,12 @@ sub _make_rv {
     return $rv;
 }
 
-sub _extract_info {
-    my ($fname) = @_;
-
-    use vars qw(%inchash @dl_shared_objects @incarray);
-
-    unless (do $fname) {
-        die "error extracting info from DataFeed file: ",
-            $@ || "can't read $fname: $!";
-    }
-
-    my %ih = %inchash;
-    my @dso = @dl_shared_objects;
-    my @ia = @incarray;
-    return (\%ih, \@dso, \@ia);
-}
-
 sub _gettype {
     my $name = shift;
-    my $dlext = quotemeta(dl_ext());
 
-    return 'autoload' if $name =~ /(?:\.ix|\.al)$/i;
+    return 'autoload' if $name =~ /\.(?:ix|al|bs)$/i;
     return 'module'   if $name =~ /\.p[mh]$/i;
-    return 'shared'   if $name =~ /\.$dlext$/i;
+    return 'shared'   if $name =~ /\.\Q$Config{dlext}\E$/i;
     return 'data';
 }
 
@@ -1525,8 +1571,8 @@ sub _merge_rv {
         my %mark;
         if ($rv->{$key} and _not_dup($key, $rv, $rv_sub)) {
             warn "Different modules for file '$key' were found.\n"
-                . " -> Using '" . _abs_path($rv_sub->{$key}{file}) . "'.\n"
-                . " -> Ignoring '" . _abs_path($rv->{$key}{file}) . "'.\n";
+                . " -> Using '" . abs_path($rv_sub->{$key}{file}) . "'.\n"
+                . " -> Ignoring '" . abs_path($rv->{$key}{file}) . "'.\n";
             $rv->{$key}{used_by} = [
                 grep (!$mark{$_}++,
                     @{ $rv->{$key}{used_by} },
@@ -1559,22 +1605,13 @@ sub _merge_rv {
 sub _not_dup {
     my ($key, $rv1, $rv2) = @_;
     if (File::Spec->case_tolerant()) {
-        return lc(_abs_path($rv1->{$key}{file})) ne lc(_abs_path($rv2->{$key}{file}));
+        return lc(abs_path($rv1->{$key}{file})) ne lc(abs_path($rv2->{$key}{file}));
     }
     else {
-        return _abs_path($rv1->{$key}{file}) ne _abs_path($rv2->{$key}{file});
+        return abs_path($rv1->{$key}{file}) ne abs_path($rv2->{$key}{file});
     }
 }
 
-sub _abs_path {
-    return join(
-        '/',
-        Cwd::abs_path(File::Basename::dirname($_[0])),
-        File::Basename::basename($_[0]),
-    );
-}
-
-
 sub _warn_of_runtime_loader {
     my $module = shift;
     return if $SeenRuntimeLoader{$module}++;
@@ -1634,6 +1671,10 @@ scripts that contains prerequisite modules; this module supports two
 such projects, L<PAR> and L<App::Packer>.  Please see their respective
 documentations on CPAN for further information.
 
+Other modules which accomplish the same goal with different approach:
+L<Module::ExtractUse>, L<Perl::PrereqScanner>, L<Perl::PrereqScanner::Lite>,
+L<Perl::PrereqScanner::NotQuiteLite>.
+
 =head1 AUTHORS
 
 Audrey Tang E<lt>cpan@audreyt.orgE<gt>
